function buildUI(tool){
    tool.addParameterLink("mesh", true);
	tool.addParameterButton("apply","OK","apply");
}

function apply(tool) {
	var document = tool.document();
	var x = tool.getParameter("mesh");
	
	var childCount = x.childCount();
	var core = x.core();
	var particleCount = core.particleCount();
	print("particleCount: " + particleCount);
	var j = 0;
	folder = document.addObject(FOLDER);
	folder.setParameter("name", "Particle Mesh Group");
	folder.setParameter("position", x.getParameter("position"));
	folder.setParameter("rotation", x.getParameter("rotation"));
	folder.setParameter("scale", x.getParameter("scale"));

	for (var i = 0; i < particleCount; i++) {
		var particle = core.particleAtIndex(i);
		
		subfolder = document.addObject(FOLDER);
		folder.addChildAtIndex(subfolder, i);
		
		subfolder.setParameter("position", particle.getPosition());
		var rotation = particle.getRotation();
		subfolder.setParameter("rotation", new Vec3D(rotation.y, rotation.x, rotation.z));
		//subfolder.setParameter("scale", particle.getScale());
		
		child = x.childAtIndex(j);
		copy = copyObject(document, child);
		subfolder.addChildAtIndex(copy, 0);
		
		j++;
		if (j == childCount) {
			print("RESET");
			j = 0;
		}
		print("i: " + i + " j: " + j)
	}
}

function copyObject (document, object) {
	var copy = document.addObject(object.type());
	
	// Copy Parameters
	var name     = object.getParameter("name");
	var position = object.getParameter("position");
	var rotation = object.getParameter("rotation");
	var scale    = object.getParameter("scale");
	copy.setParameter("name",name);
	copy.setParameter("position",position);
	copy.setParameter("rotation",rotation);
	copy.setParameter("scale",scale);
	
	// Copy Materials
	var array = object.materialTags();
	var count = array.length;
	for (var index = 0; index < count; index++) {
		var ID  = array[index].linkedToMaterial();
		var mat = document.materialAtIndex(ID);
		var tag = copy.addTagOfType(SHADERTAG);
		tag.setParameter("material",mat);
	}
	
	// Copy Geometry
	if (object.type() == POLYGONOBJ) {
		var count = object.core().vertexCount();
		for (var index = 0; index < count; index++) {
			var vertex = object.core().vertex(index);
			copy.core().addVertex(false, vertex);
		}
		var count = object.core().polygonCount();
		for (var index = 0; index < count; index++) {
			var verts = [];
			var uvs   = [];
			var size  = object.core().polygonSize(index);
			for (var corner = 0; corner < size; corner++) {
				var vertex = object.core().vertexIndex(index,corner);
				var uv     = object.core().uvCoord(index,corner);
				verts[corner] = vertex;
				uvs[corner]   = new Vec2D(uv.x,uv.y);
			}
			copy.core().addIndexPolygon(size,verts,uvs);
		}
	}
	return copy;
}